@@ -92,8 +92,8 @@ impl Encoder for TTYEncoder {
                     write!(out, "\x1b[r")?;
                 }
             }
-            Image(_) | ImageErase(_) => {
-                // image is ignored and must be handled by image handler
+            Image(_, _) | ImageErase(_, _) => {
+                // image commands are ignored and must be handled by image handler
             }
             Termcap(caps) => {
                 write!(out, "\x1bP+q")?;
@@ -123,13 +123,6 @@ impl Encoder for TTYEncoder {
             Title(title) => {
                 write!(out, "\x1b]0;{}\x1b\\", title)?;
             }
-            SynchronizeUpdate(enable) => {
-                if enable {
-                    write!(out, "\x1bP=1s\x1b\\")?;
-                } else {
-                    write!(out, "\x1bP=2s\x1b\\")?;
-                }
-            }
             DeviceAttrs => {
                 write!(out, "\x1b[c")?;
             }
